﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace DWZB_Agv_Agreement
{

    #region 自定义委托
    /// <summary>
    /// 获取消息委托
    /// </summary>
    /// <param name="sender">消息主体buffer</param>
    /// <param name="ip">ip</param>
    /// <param name="dataLength">数据长度</param>
    /// <param name="e">事件类对象</param>
    public delegate void GetMessageEventHandler(object sender, string ip, int dataLength, EventArgs e);
    /// <summary>
    /// 获取链接委托
    /// </summary>
    /// <param name="sender">链接进入的IP</param>
    /// <param name="e">事件类对象</param>
    public delegate void GetAcceptEventHandler(object sender, EventArgs e);
    /// <summary>
    /// 关闭链接委托
    /// </summary>
    /// <param name="sender">链接进入的IP</param>
    /// <param name="e">事件类对象</param>
    public delegate void GetCloseEventHandler(object sender, EventArgs e); 
    #endregion

    public class SocketServerHelper
    {

        #region 自定义事件
        public event GetMessageEventHandler MessageEvent;
        /// <summary>
        /// 触发接收到Socket客户端消息的委托事件
        /// </summary>
        /// <param name="sender">数据buffer</param>
        /// <param name="ip">ip</param>
        /// <param name="dataLength">数据长度</param>
        /// <param name="e">事件类对象</param>
        protected virtual void OnNewMessage(object sender, string ip, int dataLength, EventArgs e)
        {
            MessageEvent?.Invoke(sender, ip, dataLength, e);
        }


        public event GetAcceptEventHandler AcceptEvent;
        /// <summary>
        /// 触发Socket客户接入服务端的委托事件
        /// </summary>
        /// <param name="sender">链接进入的IP（Socket对象）</param>
        /// <param name="e">事件类对象</param>
        protected virtual void OnAcceptEvent(object sender, EventArgs e)
        {
            AcceptEvent?.Invoke(sender, e);
        }

        public event GetCloseEventHandler CloseEvent;
        /// <summary>
        /// 触发关闭链接委托事件
        /// </summary>
        /// <param name="sender">链接进入的IP（Socket对象）</param>
        /// <param name="e">事件类对象</param>
        protected virtual void OnCloseEvent(object sender, EventArgs e)
        {
            CloseEvent?.Invoke(sender, e);
        }

        #endregion

        #region 全局变量
        /// <summary>
        /// Socket服务端套接字对象
        /// </summary>
        public Socket SocketServerListen;
        /// <summary>
        /// Socket服务端监控可取消线程对象
        /// </summary>
        public BackgroundWorker SocketServerThWorker;
        /// <summary>
        /// Socket客户端套接字对象字典
        /// </summary>
        public ConcurrentDictionary<string, Socket> dicSockClientList = new ConcurrentDictionary<string, Socket>();
        /// <summary>
        /// Socket客户端接收数据线程字典
        /// </summary>
        public ConcurrentDictionary<string, BackgroundWorker> dicSockThreadWorkerList = new ConcurrentDictionary<string, BackgroundWorker>();
        #endregion

        #region 方法
        /// <summary>
        /// 开启监听
        /// </summary>
        /// <param name="ip">服务端IP地址</param>
        /// <param name="port"服务端>端口号</param>
        public void StartListen(string ip, int port)
        {
            try
            {
                //当开始监听的时候，在服务器端创建一个负责监听IP地址跟端口号的Socket
                SocketServerListen = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                //获取ip地址
                IPAddress ipadd = IPAddress.Parse(ip);
                //创建一个网络通信节点，这个通信节点包含了ip地址，端口号。
                //这里的端口我们设置为1029，这里设置大于1024，为什么自己查一下端口号范围使用说明。
                IPEndPoint endpoint = new IPEndPoint(ipadd, Convert.ToInt32(port));//创建一个网络通信节点，该节点中包含了IP地址和端口号.
                //Socket绑定网络通信节点
                SocketServerListen.Bind(endpoint);
                //设置监听队列
                SocketServerListen.Listen(100);
                LogHelper.WriteLog($"Socket服务端 ip地址：【{ip}】端口号：【{port}】 开启监听成功！", LogTypeEnum.SocketRun);
            }
            catch (Exception ex)
            {
                LogHelper.WriteLog($"Socket服务端 ip地址：【{ip}】端口号：【{port}】 开启监听发生异常：【{ex.Message}】", LogTypeEnum.SocketErr);
            }
        }

        #region UDP协议
        //public void StartListenUdp(string ip, int port, List<BasBarcodeScannerMd> allBarcodeScannerList)
        //{
        //    try
        //    {
        //        //当开始监听的时候，在服务器端创建一个负责监听IP地址跟端口号的Socket
        //        SocketServerListen = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
        //        //获取ip地址
        //        IPAddress ipadd = IPAddress.Parse(ip);
        //        //创建一个网络通信节点，这个通信节点包含了ip地址，端口号。
        //        //这里的端口我们设置为1029，这里设置大于1024，为什么自己查一下端口号范围使用说明。
        //        IPEndPoint endpoint = new IPEndPoint(ipadd, Convert.ToInt32(port));//创建一个网络通信节点，该节点中包含了IP地址和端口号.
        //        //Socket绑定网络通信节点
        //        SocketServerListen.Bind(endpoint);
        //        LogHelper.WriteLog($"Socket服务端 ip地址：【{ip}】端口号：【{port}】 开启监听成功！", LogTypeEnum.SocketRun);
        //        ParameterizedThreadStart par = new ParameterizedThreadStart(ReciveMsg);
        //        Thread t = new Thread(par)
        //        {
        //            IsBackground = true
        //        };
        //        t.Start(allBarcodeScannerList);
        //    }
        //    catch (Exception ex)
        //    {
        //        LogHelper.WriteLog($"Socket服务端 ip地址：【{ip}】端口号：【{port}】 开启监听发生异常：【{ex.Message}】", LogTypeEnum.SocketErr);
        //    }
        //}
        //public void ReciveMsg(object allBarcodeScannerList)
        //{
        //    List<BasBarcodeScannerMd> scannerList = allBarcodeScannerList as List<BasBarcodeScannerMd>;
        //    while (true)
        //    {
        //        try
        //        {
        //            EndPoint point = new IPEndPoint(IPAddress.Any, 0);
        //            byte[] buffer = new byte[1024 * 1024];
        //            int length = SocketServerListen.ReceiveFrom(buffer, ref point);
        //            if (scannerList.FindAll(x => x.IpAddress == ((IPEndPoint)point).Address.ToString()).Count > 0)
        //            {
        //                //string message = Encoding.UTF8.GetString(buffer, 0, length);
        //                //触发接收到Socket客户端发送数据的事件
        //                OnNewMessage(buffer, point.ToString(), length, EventArgs.Empty);
        //            }
        //        }
        //        catch
        //        {

        //        }
        //    }
        //}
        #endregion

        public void StartListenClientAccept()
        {
            SocketServerThWorker = new BackgroundWorker
            {
                WorkerReportsProgress = true,
                WorkerSupportsCancellation = true
            };
            SocketServerThWorker.DoWork += (object sender, DoWorkEventArgs e) => {
                while (true)
                {
                    if (SocketServerThWorker.CancellationPending)
                    {
                        e.Cancel = true;
                        return;
                    }
                    try
                    {
                        Socket newSoket = SocketServerListen.Accept();
                        string ipAddress = newSoket.RemoteEndPoint.ToString();
                        BackgroundWorker ThWorkerMd = new BackgroundWorker
                        {
                            WorkerReportsProgress = true,
                            WorkerSupportsCancellation = true
                        };
                        ThWorkerMd.DoWork += (object sender, DoWorkEventArgs e) => {
                            Socket newSocket = e.Argument as Socket;//转成对应的Socket类型
                            string IPPortID = newSocket.RemoteEndPoint.ToString();//这个线程ID
                            while (true)
                            {
                                if (ThWorkerMd.CancellationPending)
                                {
                                    e.Cancel = true;
                                    break;
                                }
                                byte[] buffer = new byte[1024 * 2];
                                int receiveLength;
                                try  //由于Socket中的Receive方法容易抛出异常，所以我们在这里要捕获异常。
                                {
                                    bool a = dicSockClientList[IPPortID].Poll(10, SelectMode.SelectRead);
                                    receiveLength = dicSockClientList[IPPortID].Receive(buffer);//接收从客户端发送过来的数据
                                    if (a)
                                    {
                                        LogHelper.WriteLog($"Socket客户端：【{IPPortID}】主动断开连接，Socket服务端准备移除该客户端连接！", LogTypeEnum.SocketErr);
                                        //触发Socket客户端断开连接实践
                                        OnCloseEvent(IPPortID, EventArgs.Empty);
                                        break;
                                    }
                                }
                                catch (Exception ex)
                                {
                                    LogHelper.WriteLog($"Socket客户端：【{IPPortID}】数据接受发生异常：【{ex.Message}】，Socket服务端已经移除该客户端连接！", LogTypeEnum.SocketErr);
                                    //触发Socket客户端断开连接实践
                                    OnCloseEvent(IPPortID, EventArgs.Empty);
                                    break;
                                }
                                if (receiveLength > 0)
                                {
                                    //记录接收到的原始数据                    
                                    LogHelper.WriteLog($"Socket客户端：【{IPPortID}】接收到：【{receiveLength}】字节数据，数据内容：【{BitConverter.ToString(buffer, 0, receiveLength)}】。", LogTypeEnum.SocketData);
                                    //触发接收到Socket客户端发送数据的事件
                                    OnNewMessage(buffer, IPPortID, receiveLength, EventArgs.Empty);
                                }
                            }
                        };
                        ThWorkerMd.RunWorkerAsync(newSoket);

                        dicSockClientList.TryAdd(ipAddress, newSoket);
                        dicSockThreadWorkerList.TryAdd(ipAddress, ThWorkerMd);
                        //触发Socket客户端接入服务端事件
                        OnAcceptEvent(ipAddress, EventArgs.Empty);

                        LogHelper.WriteLog($"Socket客户端:【{ipAddress}】连接成功!", LogTypeEnum.SocketRun);
                    }
                    catch (Exception ex)
                    {
                        LogHelper.WriteLog($"Socket服务端监听客户端的连接的函数发生异常：【{ex.Message}】", LogTypeEnum.SocketErr);
                    }
                }
            };
            SocketServerThWorker.RunWorkerAsync();
        }

        /// <summary>
        /// 循环监听客户端的连接
        /// </summary>
        public void ListeneSocketClientConnectAccept()
        {
            while (true)
            {
                try
                {
                    Socket newSoket = SocketServerListen.Accept();
                    string ipAddress = newSoket.RemoteEndPoint.ToString();

                    ParameterizedThreadStart par = new(RecevieMsg);
                    Thread thread = new(par)
                    {
                        IsBackground = true
                    };
                    thread.Start(newSoket);

                    dicSockClientList.TryAdd(ipAddress, newSoket);
                    //dicSockThreadList.TryAdd(ipAddress, thread);
                    //触发Socket客户端接入服务端事件
                    OnAcceptEvent(ipAddress, EventArgs.Empty);

                    LogHelper.WriteLog($"Socket客户端:【{ipAddress}】连接成功!", LogTypeEnum.SocketRun);
                }
                catch (Exception ex)
                {
                    LogHelper.WriteLog($"Socket服务端监听客户端的连接的函数发生异常：【{ex.Message}】", LogTypeEnum.SocketErr);
                }
            }
        }

        /// <summary>
        /// //该方法负责接收从客户端发送过来的数据
        /// </summary>
        /// <param name="socket">Socket套接字</param>
        public void RecevieMsg(object socket)
        {
            Socket newSocket = socket as Socket;//转成对应的Socket类型
            string IPPortID = newSocket.RemoteEndPoint.ToString();//这个线程ID

            while (true)
            {
                byte[] buffer = new byte[1024 * 2];
                int receiveLength;
                try  //由于Socket中的Receive方法容易抛出异常，所以我们在这里要捕获异常。
                {
                    bool a = dicSockClientList[IPPortID].Poll(10,SelectMode.SelectRead);
                    receiveLength = dicSockClientList[IPPortID].Receive(buffer);//接收从客户端发送过来的数据
                    if (a)
                    {
                        LogHelper.WriteLog($"Socket客户端：【{IPPortID}】主动断开连接，Socket服务端准备移除该客户端连接！", LogTypeEnum.SocketErr);
                        //触发Socket客户端断开连接实践
                        OnCloseEvent(IPPortID, EventArgs.Empty);
                        break;
                    }
                }
                catch (Exception ex)
                {                    
                    LogHelper.WriteLog($"Socket客户端：【{IPPortID}】数据接受发生异常：【{ex.Message}】，Socket服务端已经移除该客户端连接！", LogTypeEnum.SocketErr);
                    //触发Socket客户端断开连接实践
                    OnCloseEvent(IPPortID, EventArgs.Empty);
                    break;
                }
                if (receiveLength > 0)
                {
                    //记录接收到的原始数据                    
                    LogHelper.WriteLog($"Socket客户端：【{IPPortID}】接收到：【{receiveLength}】字节数据，数据内容：【{BitConverter.ToString(buffer, 0, receiveLength)}】。", LogTypeEnum.SocketData);
                    //触发接收到Socket客户端发送数据的事件
                    OnNewMessage(buffer, IPPortID, receiveLength, EventArgs.Empty);
                }
            }
        }

        /// <summary>
        /// 关闭服务
        /// </summary>
        public void CloseServer()
        {
            if (SocketServerThWorker.IsBusy)
            {
                SocketServerThWorker.CancelAsync();
            }

            lock (dicSockThreadWorkerList)
            {
                foreach (var item in dicSockThreadWorkerList)
                {
                    if (item.Value.IsBusy)
                    {
                        item.Value.CancelAsync();
                    }
                }
                dicSockThreadWorkerList.Clear();//清除字典
            }

            lock (dicSockClientList)
            {
                foreach (var item in dicSockClientList)
                {
                    item.Value.Disconnect(true);
                    item.Value.Close();//关闭每一个连接
                }
                dicSockClientList.Clear();//清除字典
            }

            if (SocketServerListen != null)
            {
                SocketServerListen.Close();
                SocketServerListen.Dispose();
            }
            SocketServerThWorker.Dispose();
        }

        public void SendData(string ipAddress,byte[] btArr)
        {
            dicSockClientList[ipAddress].Send(btArr);
        }

        public void RemoveSocketClient(string ipAddress)
        {
            dicSockClientList[ipAddress].Disconnect(true);
            dicSockClientList[ipAddress].Close();//关闭每一个连接
            dicSockClientList.TryRemove(ipAddress, out _);
            dicSockThreadWorkerList[ipAddress].CancelAsync();
            dicSockThreadWorkerList.TryRemove(ipAddress, out _);
        }
        #endregion

    }
}
